@@ -1,4 +1,4 @@
-# -*- coding: UTF-8 -*-
+# -*- coding: utf-8 -*-
 #
 # 
 # This is part of Yappy
@@ -6,7 +6,7 @@
 #
 # demo.py -- some simple parsers
 #
-# Copyright (C) 2000-2003 Rogrio Reis & Nelma Moreira {rvr,nam}@ncc.up.pt
+# Copyright (C) 2000-2003 Rogério Reis & Nelma Moreira {rvr,nam}@ncc.up.pt
 #
 
 from yappy.parser import *
@@ -366,6 +366,6 @@ if __name__ == '__main__':
     st = "2-24*9"
     print "Input:", st
     print "Result:", d.input(st)
-    st = "2-24*9-34*2+1"
+    st = "2-24K*9-34*2+1"
     print "Input:", st
     print "Result:", d.input(st)
