@@ -1,4 +1,4 @@
-# -*- coding: UTF-8 -*-
+# -*- coding: utf-8 -*-
 """
 This is part of Yappy
 
@@ -10,7 +10,7 @@ A LR parser generator, based on Aho and al. 1986, C{Compilers}
 It currently builds C{SLR}, C{LR(1)} and  C{LALR(1)} parsing tables.
 
 Copyright (C) 2000-2003 Rogério Reis & Nelma Moreira {rvr,nam}@ncc.up.pt
-Version: $Id: parser.py,v 1.8 2004/03/26 15:33:29 nam Exp $
+Version: $Id: parser.py,v 1.9 2004/12/02 10:17:50 nam Exp $
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -35,19 +35,22 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 """
 
 from types import *
-import pcre, re, exceptions
+import re, exceptions
 import sys, string, copy, time, operator
 
 import shelve
 
+# set elements are mutable objects; we cannot use sets
 import osets
 
 #Globals
 
-_DEBUG = 1
+_DEBUG = 0
 
 _Version = 1.2
 
+NIL = ""
+
 class Lexer:
     """Class for lexical analyser to use with the parser
 
@@ -86,7 +89,7 @@ class Lexer:
             rex = r[0]
             funct = r[1]
             try: rec  = re.compile(rex)
-            except pcre.error:
+            except TypeError:
                 raise LexicalRulesErrorRE(rex,rnumber)
             try:
                 op,prec,assoc = r[2]
@@ -747,7 +750,7 @@ class LRtable:
         
         """
         try:
-            if self.operators.has_key(a):
+            if self.operators and self.operators.has_key(a):
                 prec_op, assoc_op = self.operators[a]
             else:
                 prec_op, assoc_op  = (0,'DUMMY')
@@ -758,7 +761,7 @@ class LRtable:
                 else:
                     self.ACTION[(i,a)] = ('shift',s)
                     if _DEBUG: print "solved shift %s" %s
-        except:
+        except (AttributeError, TypeError, KeyError,NameError):
             if self.Log.noconflicts:
                 # choose to shift
                 self.ACTION[(i,a)] = ('shift',s)
@@ -1362,7 +1365,7 @@ class LRparser:
                     raise TableError(table_shelve)
             try:
                 self.Log = d['log']
-            except:
+            except KeyError:
                 raise TableError(table_shelve)
         else:    
             self.table = tabletype(self.cfgr,operators,noconflicts,expect)
@@ -1446,18 +1449,18 @@ class LRparser:
         self.tokens.append((self.endmark,self.endmark))
         self.context = context
         self.output = []
-        ip = 0
+        self.ip = 0
         while 1:
             s = self.stack.top()[0]
-            a = self.tokens[ip][0]
+            a = self.tokens[self.ip][0]
             if _DEBUG:
-                print "Input: %s\nState: %s" %(map(lambda x:x[0],self.tokens[ip:]),s)
+                print "Input: %s\nState: %s" %(map(lambda x:x[0],self.tokens[self.ip:]),s)
                 print "Stack: %s" %self.stack
             try:    
                 if self.ACTION[s,a][0] == 'shift':
                     if _DEBUG: print "Action: shift\n" 
-                    self.stack.push((self.ACTION[s,a][1], self.tokens[ip][1]))
-                    ip = ip + 1
+                    self.stack.push((self.ACTION[s,a][1], self.tokens[self.ip][1]))
+                    self.ip = self.ip + 1
                 elif  self.ACTION[s,a][0] == 'reduce':
                     n =  self.ACTION[s,a][1]
                     if _DEBUG: print "Action: reduce %s %s\n" %(n,str(self.rules[n]))
@@ -1473,7 +1476,7 @@ class LRparser:
                     break
                 else:
                     raise LRParserError(s,a)
-            except:
+            except KeyError:
                 if _DEBUG: print "Error in action: %s" %self.ACTION
                 raise LRParserError(s,a)
         return  self.stack.top()[1]
@@ -1600,16 +1603,16 @@ class LRBuildparser:
         self.input = tokens
         self.input.append(self.table.gr.endmark)
         self.output = []
-        ip = 0
+        self.ip = 0
         while 1:
             s = self.stack.top()
-            a = self.input[ip]
+            a = self.input[self.ip]
             if not self.table.ACTION.has_key((s,a)):
-                raise LRParserError()
+                raise LRParserError(s,a)
             elif self.table.ACTION[s,a][0] == 'shift':
                 # self.stack.push(a)
                 self.stack.push(self.table.ACTION[s,a][1])
-                ip = ip + 1
+                self.ip = self.ip + 1
             elif  self.table.ACTION[s,a][0] == 'reduce':
                 n =  self.table.ACTION[s,a][1]
                 for i in range(len(self.table.gr.rules[n][1])):
@@ -1618,7 +1621,7 @@ class LRBuildparser:
                 a = self.table.gr.rules[n][0]
                 #                self.stack.push(a)
                 if not self.table.GOTO.has_key((s1,a)):
-                    raise LRParserError()
+                    raise LRParserError(s1,a)
                 else:
                     self.stack.push(self.table.GOTO[s1,a])
                     self.output.append(n)
@@ -1852,6 +1855,7 @@ class Yappy_grammar(Yappy):
         self.input(st,{'locals':locals()})
         return self.context['rules'] 
 
+        
 class Stack:
     """ A simple class to implement stacks"""
     
